/* ================================================================================== */
/* Copyright (c) 1998-1999 3Com Corporation or its subsidiaries. All rights reserved. */
/* ================================================================================== */

#include "EmulatorCommon.h"
#include "AppPreferences.h"

#include "CPU_REG.h"			// Emulator::kUnspecified



// ===========================================================================
//		 AppPreferences
// ===========================================================================

AppPreferences*	gPrefs;


// ---------------------------------------------------------------------------
//		 AppPreferences::Initialize
// ---------------------------------------------------------------------------

void AppPreferences::Initialize (void)
{
	fScale							= 2;
	fScreenBackground				= kGrayBackground;
	fWindowFrame					= kCaseFrame;
	fCloseAction					= kSaveAsk;
	fCommPort.erase ();
	fWindowLocation.h				= -1;	// Default location (centered)
	fWindowLocation.v				= -1;
	fGCWLocation.h					= -1;	// Default location (centered)
	fGCWLocation.v					= -1;

	fRedirectNetLib					= false;

	fReportCorruptedHeap			= true;		// Always on
	fReportFreeChunkAccess			= true;
	fReportHardwareRegisterAccess	= true;
	fReportInvalidPC				= true;		// Always on
	fReportLowMemoryAccess			= true;
	fReportLowStackAccess			= true;
	fReportMemMgrDataAccess			= true;
	fReportMemMgrSemaphore			= true;
	fReportScreenAccess				= true;
	fReportStackAlmostOverflow		= true;
	fReportStackOverflow			= true;		// Always on
	fReportStorageHeapAccess		= true;
	fReportSysFatalAlert			= true;		// Always on
	fReportSystemGlobalAccess		= true;
	fReportUnhandledException		= true;		// Always on
	fReportUnimplementedTrap		= true;		// Always on
	fReportUninitializedChunkAccess	= true;
	fReportUninitializedStackAccess	= true;
	fReportUnlockedChunkAccess		= true;

	fLogErrorMessages				= 0;
	fLogWarningMessages				= 0;
	fLogGremlins					= 0;
	fLogCPUOpcodes					= 0;
	fLogEnqueuedEvents				= 0;
	fLogDequeuedEvents				= 0;
	fLogSystemCalls					= 0;
	fLogApplicationCalls			= 0;
	fLogSerial						= 0;
	fLogSerialData					= 0;
	fLogNetLib						= 0;
	fLogNetLibData					= 0;
	fLogExgMgr						= 0;
	fLogExgMgrData					= 0;
	fLogHLDebugger					= 0;
	fLogHLDebuggerData				= 0;
	fLogLLDebugger					= 0;
	fLogLLDebuggerData				= 0;

	fReportMemoryLeaks				= true;
	fReportLockedRecords			= true;

	fFillNewBlocks					= false;
	fFillResizedBlocks				= false;
	fFillDisposedBlocks				= false;
	fFillStack						= false;

//	fLastConfiguration	= ...;	// Self-initializing

	if (!gPrefs)
		gPrefs = this;
}


// ---------------------------------------------------------------------------
//		 AppPreferences::ValidateSettings
// ---------------------------------------------------------------------------

Bool AppPreferences::ValidateSettings (void)
{
	Bool	changed = false;

	// Make sure fScale is in range.

	if (fScale < 1)
	{
		fScale = 1;
		changed = true;
	}
	else if (fScale > 2)
	{
		fScale = 2;
		changed = true;
	}

	// Make sure fCloseAction is in range.
	// (The local int prevents egcs from optimising out this comparison
	// on the grounds that the "comparison is always 0 due to limited
	// range of data type".  For some reason a typecast is not sufficient.)
	
	int closeAction = (int)fCloseAction;
	if (closeAction < (int)kSaveAlways)
	{
		fCloseAction = kSaveAlways;
		changed = true;
	}
	else if (fCloseAction > kSaveNever)
	{
		fCloseAction = kSaveNever;
		changed = true;
	}

	// Make sure fScreenBackground is in range.
	// (The local int prevents egcs from optimising out this comparison
	// on the grounds that the "comparison is always 0 due to limited
	// range of data type".  For some reason a typecast is not sufficient.)

	int screenBackground = (int)fScreenBackground;
	if (screenBackground < (int) kGrayBackground)
	{
		fScreenBackground = kGrayBackground;
		changed = true;
	}
	else if (fScreenBackground > kWhiteBackground)
	{
		fScreenBackground = kWhiteBackground;
		changed = true;
	}

	// Make sure fDevice is valid.

	if (fLastConfiguration.fDeviceType >= kDeviceLast)
	{
		fLastConfiguration.fDeviceType = kDeviceUnspecified;

		// We've changed the device; zap the ROM file so that we
		// don't try to load it into a possibly incompatible device.

		fLastConfiguration.fROMFile = FileReference ();
		fLastRAMFile = FileReference ();

		changed = true;
	}

	// Make sure fSilkscreenType is valid.

	if (fLastConfiguration.fSilkscreenType >= kSilkscreenLast)
	{
		fLastConfiguration.fSilkscreenType = kSilkscreenUnspecified;
		changed = true;
	}

	// Make sure fDevice is valid.

	Bool	validDevice =
		fLastConfiguration.fDeviceType == kDevicePilot1000 ||
		fLastConfiguration.fDeviceType == kDevicePilot5000 ||
		fLastConfiguration.fDeviceType == kDevicePalmPilotPersonal ||
		fLastConfiguration.fDeviceType == kDevicePalmPilotProfessional ||
		fLastConfiguration.fDeviceType == kDevicePalmIII ||
		fLastConfiguration.fDeviceType == kDevicePalmIIIx ||
		fLastConfiguration.fDeviceType == kDevicePalmV ||
		fLastConfiguration.fDeviceType == kDevicePalmVII;


	if (!validDevice)
	{
		fLastConfiguration.fDeviceType = kDevicePalmIII;

		// We've changed the device; zap the ROM file so that we
		// don't try to load it into a possibly incompatible device.

		fLastConfiguration.fROMFile = FileReference ();
		fLastRAMFile = FileReference ();

		changed = true;
	}

	// Make sure fSilkscreenType is valid.

	Bool	validSilkscreen = fLastConfiguration.fSilkscreenType == kSilkscreenEnglish;

	if (!validSilkscreen)
	{
		validSilkscreen = fLastConfiguration.fSilkscreenType == kSilkscreenJapanese;
	}

	if (!validSilkscreen)
	{
		fLastConfiguration.fSilkscreenType = kSilkscreenUnspecified;
		changed = true;
	}

	// Scale back any extra files.

	while (fPRC_MRU.size () > MRU_COUNT)
	{
		fPRC_MRU.pop_back ();
		changed = true;
	}

	while (fRAM_MRU.size () > MRU_COUNT)
	{
		fRAM_MRU.pop_back ();
		changed = true;
	}

	return changed;
}


// ---------------------------------------------------------------------------
//		 AppPreferences::UseLoggingOptions
// ---------------------------------------------------------------------------

int AppPreferences::UseLoggingOptions (int newType)
{
	int	oldType = fCurLoggingType;
	fCurLoggingType = newType;

#define UPDATE(x)					\
	if (((x) & (newType)) != 0)		\
		(x) |=  kCurrentLogging;	\
	else							\
		(x) &=  ~kCurrentLogging

	UPDATE (fLogErrorMessages);
	UPDATE (fLogWarningMessages);
	UPDATE (fLogGremlins);
	UPDATE (fLogCPUOpcodes);
	UPDATE (fLogEnqueuedEvents);
	UPDATE (fLogDequeuedEvents);
	UPDATE (fLogSystemCalls);
	UPDATE (fLogApplicationCalls);
	UPDATE (fLogSerial);
	UPDATE (fLogSerialData);
	UPDATE (fLogNetLib);
	UPDATE (fLogNetLibData);
	UPDATE (fLogExgMgr);
	UPDATE (fLogExgMgrData);
	UPDATE (fLogHLDebugger);
	UPDATE (fLogHLDebuggerData);
	UPDATE (fLogLLDebugger);
	UPDATE (fLogLLDebuggerData);

#undef UPDATE

	return oldType;
}


// ---------------------------------------------------------------------------
//		 AppPreferences::GetCommPort
// ---------------------------------------------------------------------------

void AppPreferences::GetCommPort (unsigned char* result)
{
	string	portName = AppPreferences::GetCommPort ();
	result[0] = portName.length ();
	memcpy ((char*) &result[1], portName.c_str (), result[0]);
}


// ---------------------------------------------------------------------------
//		 AppPreferences::UpdatePRCMRU
// ---------------------------------------------------------------------------

void AppPreferences::UpdatePRCMRU (const FileReference& newFile)
{
	UpdateMRU (fPRC_MRU, newFile);
}


// ---------------------------------------------------------------------------
//		 AppPreferences::UpdateRAMMRU
// ---------------------------------------------------------------------------

void AppPreferences::UpdateRAMMRU (const FileReference& newFile)
{
	UpdateMRU (fRAM_MRU, newFile);
}


// ---------------------------------------------------------------------------
//		 AppPreferences::UpdateMRU
// ---------------------------------------------------------------------------

void AppPreferences::UpdateMRU (FileRefList& fileList, const FileReference& newFile)
{
	RemoveMRU (fileList, newFile);

	fileList.push_front (newFile);

	while (fileList.size () > MRU_COUNT)
		fileList.pop_back ();
}


// ---------------------------------------------------------------------------
//		 AppPreferences::RemovePRCMRU
// ---------------------------------------------------------------------------

void AppPreferences::RemovePRCMRU (const FileReference& oldFile)
{
	RemoveMRU (fPRC_MRU, oldFile);
}


// ---------------------------------------------------------------------------
//		 AppPreferences::RemoveRAMMRU
// ---------------------------------------------------------------------------

void AppPreferences::RemoveRAMMRU (const FileReference& oldFile)
{
	RemoveMRU (fRAM_MRU, oldFile);
}


// ---------------------------------------------------------------------------
//		 AppPreferences::RemoveMRU
// ---------------------------------------------------------------------------

void AppPreferences::RemoveMRU (FileRefList& fileList, const FileReference& oldFile)
{
	FileRefList::iterator	iter = find (fileList.begin (), fileList.end (), oldFile);

	if (iter != fileList.end ())
		fileList.erase (iter);
}


// ---------------------------------------------------------------------------
//		 AppPreferences::GetPRCMRU
// ---------------------------------------------------------------------------

FileReference& AppPreferences::GetPRCMRU (int index)
{
	if (index < fPRC_MRU.size ())
		return fPRC_MRU [index];

	static FileReference	emptyRef;

	return emptyRef;
}


// ---------------------------------------------------------------------------
//		 AppPreferences::GetRAMMRU
// ---------------------------------------------------------------------------

FileReference& AppPreferences::GetRAMMRU (int index)
{
	if (index < fRAM_MRU.size ())
		return fRAM_MRU [index];

	static FileReference	emptyRef;

	return emptyRef;
}


// ---------------------------------------------------------------------------
//		 Configuration::Configuration
// ---------------------------------------------------------------------------

AppPreferences::Configuration::Configuration () :
	fDeviceType (kDeviceUnspecified),
	fSilkscreenType (kSilkscreenUnspecified),
	fRAMSize (1024),
	fROMFile ()
{
}


// ---------------------------------------------------------------------------
//		 GremlinInfo::GremlinInfo
// ---------------------------------------------------------------------------

AppPreferences::GremlinInfo::GremlinInfo () :
	fNumber (0),
	fSteps (-1),
	fSaveFrequency (10000),
	fAppList (),
	fContinuePastWarnings (false)
{
}

