/* ================================================================================== */
/* Copyright (c) 1998-1999 3Com Corporation or its subsidiaries. All rights reserved. */
/* ================================================================================== */

#include "EmulatorCommon.h"
#include "Byteswapping.h"

#if BYTESWAP

// For reasons of "convenience", the official type declarations for these
// packets omits their bodies.  For the emulator, we'd really like to have
// the bodies, so we declare these parallel types.

struct SysPktReadRegsRspType2
{
	_sysPktBodyCommon;								// Common Body header
	M68KRegsType	reg;							// <- return registers
};

struct SysPktGetBreakpointsRspType2
{
	_sysPktBodyCommon;								// Common Body header
	BreakpointType	bp[dbgTotalBreakpoints];
};

struct SysPktSetBreakpointsCmdType2
{
	_sysPktBodyCommon;								// Common Body header
	BreakpointType	bp[dbgTotalBreakpoints];
};

struct SysPktGetTrapBreaksRspType2
{
	_sysPktBodyCommon;								// Common Body header
	Word		trapBP[dbgTotalTrapBreaks];
};

struct SysPktSetTrapBreaksCmdType2
{
	_sysPktBodyCommon;								// Common Body header
	Word		trapBP[dbgTotalTrapBreaks];
};

struct SysPktGetTrapConditionsRspType2
{
	_sysPktBodyCommon;								// Common Body header
	Word		trapParam[dbgTotalTrapBreaks];
};

struct SysPktSetTrapConditionsCmdType2
{
	_sysPktBodyCommon;								// Common Body header
	Word		trapParam[dbgTotalTrapBreaks];
};


#if defined (_MSC_VER) || defined (__GNUC__)

unsigned long	flipBits[] =
{
	/* 0000 -> 0000 */	0x00,
	/* 0001 -> 1000 */	0x08,
	/* 0010 -> 0100 */	0x04,
	/* 0011 -> 1100 */	0x0C,
	/* 0100 -> 0010 */	0x02,
	/* 0101 -> 1010 */	0x0A,
	/* 0110 -> 0110 */	0x06,
	/* 0111 -> 1110 */	0x0E,
	/* 1000 -> 0001 */	0x01,
	/* 1001 -> 1001 */	0x09,
	/* 1010 -> 0101 */	0x05,
	/* 1011 -> 1101 */	0x0D,
	/* 1100 -> 0011 */	0x03,
	/* 1101 -> 1011 */	0x0B,
	/* 1110 -> 0111 */	0x07,
	/* 1111 -> 1111 */	0x0F
};

#endif

void Canonical (BreakpointType& bp)
{
	Canonical (bp.addr);

	Canonical (bp.enabled);
	Canonical (bp.installed);
}


void Canonical (CardHeaderType& cardHdr)
{
	Canonical (cardHdr.initStack);
	Canonical (cardHdr.resetVector);
	Canonical (cardHdr.signature);
	Canonical (cardHdr.hdrVersion);
	Canonical (cardHdr.flags);
	Canonical (cardHdr.version);
	Canonical (cardHdr.creationDate);
	Canonical (cardHdr.numRAMBlocks);
	Canonical (cardHdr.blockListOffset);
	Canonical (cardHdr.readWriteParmsOffset);
	Canonical (cardHdr.readWriteParmsSize);
	Canonical (cardHdr.readOnlyParmsOffset);
	Canonical (cardHdr.bigROMOffset);
	Canonical (cardHdr.checksumBytes);
	Canonical (cardHdr.checksumValue);
	Canonical (cardHdr.version);
}


void Canonical (DatabaseDirType& db)
{
	// !!! Note: this byteswapper works only in
	// one direction.  Because numDatabases is
	// converted early on, this function is assuming
	// that it is in Canonical form.  Thus, this function
	// can only be used to convert from Canonical to
	// Host form.

	Canonical (db.nextDatabaseListID);
	Canonical (db.numDatabases);

	for (Word ii = 0; ii < db.numDatabases; ++ii)
	{
		Canonical (db.databaseID[ii]);
	}
}


void Canonical (DatabaseHdrType& db)
{
	Canonical (db.attributes);
	Canonical (db.version);
	Canonical (db.creationDate);
	Canonical (db.modificationDate);
	Canonical (db.lastBackupDate);
	Canonical (db.modificationNumber);
	Canonical (db.appInfoID);
	Canonical (db.sortInfoID);
	Canonical (db.type);
	Canonical (db.creator);
	Canonical (db.uniqueIDSeed);
	Canonical (db.recordList.nextRecordListID);
	Canonical (db.recordList.numRecords);
}


void Canonical (DmAccessType& db)
{
	Canonical (db.next);
	Canonical (db.mode);
	Canonical (db.openP);
	Canonical (db.savedModNum);
}


void Canonical (DmOpenInfoType& info)
{
	Canonical (info.next);
	Canonical (info.openCount);
	Canonical (info.ownerTaskID);
//	Canonical (info.exclusive);	// !!! sort these bitfields out
//	Canonical (info.writeAccess);
//	Canonical (info.resDB);
	Canonical (info.hdrID);
	Canonical (info.hdrH);
	Canonical (info.hdrP);
	Canonical (info.cardNo);
	Canonical (info.numRecords);
	Canonical (info.hdrMasterP);
	Canonical (info.handleTableP);
}


void Canonical (DmSearchStateType& state)
{
	// This is an opaque type.  We don't have to byteswap its
	// contents, as nothing on the host will access them.  Indeed,
	// the internal structure of this data type varies from system
	// to system, and so we *can't* byteswap it without knowing
	// what version of the system we're running.
}


void Canonical (EventType& event)
{
	// Compile time assertion
	{
		char	_dummy1[offsetof(EventType, eType) == 0 ? 1 : 0];
		char	_dummy2[offsetof(EventType, penDown) == 2 ? 1 : 0];
		char	_dummy3[offsetof(EventType, screenX) == 4 ? 1 : 0];
		char	_dummy4[offsetof(EventType, screenY) == 6 ? 1 : 0];
		char	_dummy5[offsetof(EventType, data) == 8 ? 1 : 0];
		(void)_dummy1;
		(void)_dummy2;
		(void)_dummy3;
		(void)_dummy4;
		(void)_dummy5;
	}

	Canonical (event.eType);
	Canonical (event.penDown);
	Canonical (event.screenX);
	Canonical (event.screenY);

	switch(event.eType)
	{
		case nilEvent:
			break;

		case penDownEvent:
			Canonical (event.data.penUp.start);
			Canonical (event.data.penUp.end);
			break;

		case penUpEvent:
			Canonical (event.data.penUp.start);
			Canonical (event.data.penUp.end);
			break;

		case penMoveEvent:
			Canonical (event.data.penUp.start);
			Canonical (event.data.penUp.end);
			break;

		case keyDownEvent:
			Canonical (event.data.keyDown.chr);
			Canonical (event.data.keyDown.keyCode);
			Canonical (event.data.keyDown.modifiers);
			break;

		case winEnterEvent:
			Canonical (event.data.winEnter.enterWindow);
			Canonical (event.data.winEnter.exitWindow);
			break;

		case winExitEvent:
			Canonical (event.data.winExit.enterWindow);
			Canonical (event.data.winExit.exitWindow);
			break;

		case ctlEnterEvent:
			Canonical (event.data.ctlEnter.controlID);
			Canonical (event.data.ctlEnter.pControl);
			break;

		case ctlExitEvent:
			break;

		case ctlSelectEvent:
			Canonical (event.data.ctlSelect.controlID);
			Canonical (event.data.ctlSelect.pControl);
			Canonical (event.data.ctlSelect.on);
			break;

		case ctlRepeatEvent:
			Canonical (event.data.ctlRepeat.controlID);
			Canonical (event.data.ctlRepeat.pControl);
			Canonical (event.data.ctlRepeat.time);
			break;

		case lstEnterEvent:
			Canonical (event.data.lstEnter.listID);
			Canonical (event.data.lstEnter.pList);
			Canonical (event.data.lstEnter.selection);
			break;

		case lstSelectEvent:
			Canonical (event.data.lstSelect.listID);
			Canonical (event.data.lstSelect.pList);
			Canonical (event.data.lstSelect.selection);
			break;

		case lstExitEvent:
			Canonical (event.data.lstExit.listID);
			Canonical (event.data.lstExit.pList);
			break;

		case popSelectEvent:
			Canonical (event.data.popSelect.controlID);
			Canonical (event.data.popSelect.controlP);
			Canonical (event.data.popSelect.listID);
			Canonical (event.data.popSelect.listP);
			Canonical (event.data.popSelect.selection);
			Canonical (event.data.popSelect.priorSelection);
			break;

		case fldEnterEvent:
			Canonical (event.data.fldEnter.fieldID);
			Canonical (event.data.fldEnter.pField);
			break;

		case fldHeightChangedEvent:
			Canonical (event.data.fldHeightChanged.fieldID);
			Canonical (event.data.fldHeightChanged.pField);
			Canonical (event.data.fldHeightChanged.newHeight);
			Canonical (event.data.fldHeightChanged.currentPos);
			break;

		case fldChangedEvent:
			Canonical (event.data.fldChanged.fieldID);
			Canonical (event.data.fldChanged.pField);
			break;

		case tblEnterEvent:
			Canonical (event.data.tblEnter.tableID);
			Canonical (event.data.tblEnter.pTable);
			Canonical (event.data.tblEnter.row);
			Canonical (event.data.tblEnter.column);
			break;

		case tblSelectEvent:
			Canonical (event.data.tblEnter.tableID);
			Canonical (event.data.tblEnter.pTable);
			Canonical (event.data.tblEnter.row);
			Canonical (event.data.tblEnter.column);
			break;

		case daySelectEvent:
			Canonical (event.data.daySelect.pSelector);
			Canonical (event.data.daySelect.selection);
			Canonical (event.data.daySelect.useThisDate);
			break;

		case menuEvent:
			Canonical (event.data.menu.itemID);
			break;

		case appStopEvent:
			break;

		case frmLoadEvent:
			Canonical (event.data.frmLoad.formID);
			break;

		case frmOpenEvent:
			Canonical (event.data.frmOpen.formID);
			break;

		case frmGotoEvent:
			Canonical (event.data.frmGoto.formID);
			Canonical (event.data.frmGoto.recordNum);
			Canonical (event.data.frmGoto.matchPos);
			Canonical (event.data.frmGoto.matchLen);
			Canonical (event.data.frmGoto.matchFieldNum);
			Canonical (event.data.frmGoto.matchCustom);
			break;

		case frmUpdateEvent:
			Canonical (event.data.frmUpdate.formID);
			Canonical (event.data.frmUpdate.updateCode);
			break;

		case frmSaveEvent:
			break;

		case frmCloseEvent:
			Canonical (event.data.frmClose.formID);
			break;

		case frmTitleEnterEvent:
			Canonical (event.data.frmTitleEnter.formID);
			break;

		case frmTitleSelectEvent:
			Canonical (event.data.frmTitleSelect.formID);
			break;

		case tblExitEvent:
			Canonical (event.data.tblExit.tableID);
			Canonical (event.data.tblExit.pTable);
			Canonical (event.data.tblExit.row);
			Canonical (event.data.tblExit.column);
			break;

		case sclEnterEvent:
			Canonical (event.data.sclEnter.scrollBarID);
			Canonical (event.data.sclEnter.pScrollBar);
			break;

		case sclExitEvent:
			Canonical (event.data.sclExit.scrollBarID);
			Canonical (event.data.sclExit.pScrollBar);
			Canonical (event.data.sclExit.value);
			Canonical (event.data.sclExit.newValue);
			break;

		case sclRepeatEvent:
			Canonical (event.data.sclRepeat.scrollBarID);
			Canonical (event.data.sclRepeat.pScrollBar);
			Canonical (event.data.sclRepeat.value);
			Canonical (event.data.sclRepeat.newValue);
			Canonical (event.data.sclRepeat.time);
			break;
	}
}


void Canonical (FieldAttrType& attr)
{
	Canonical (*(unsigned short*) &attr);

#if defined (_MSC_VER) || defined (__GNUC__)
	unsigned short	v = *(unsigned short*) &attr;

	/*
		======== BITFIELD LAYOUT CHEAT-SHEET ========

		typedef struct {
			Word usable			:1;
			Word visible		:1;
			Word editable		:1;
			Word singleLine		:1;
			Word hasFocus       :1;
			Word dynamicSize	:1;
			Word insPtVisible	:1;
			Word dirty			:1;
			Word underlined		:2;
			Word justification	:2;
			Word autoShift		:1;
			Word hasScrollBar	:1;
			Word numeric		:1;
		} FieldAttrType;

		// On the Mac:

		|---------- high byte ----------|---------- low byte -----------|

		 15  14  13  12  11  10   9   8   7   6   5   4   3   2   1   0  
		  7   6   5   4   3   2   1   0   7   6   5   4   3   2   1   0
		+---+---+---+---+---+---+---+---+-------+-------+---+---+---+---+
		| u | v | e | s | h | d | i | d |   u   |   j   | a | h | n | * |
		+---+---+---+---+---+---+---+---+-------+-------+---+---+---+---+


		// On Windows (in-register representation):

		|---------- high byte ----------|---------- low byte -----------|

		 15  14  13  12  11  10   9   8   7   6   5   4   3   2   1   0  
		  7   6   5   4   3   2   1   0   7   6   5   4   3   2   1   0
		+---+---+---+---+-------+-------+---+---+---+---+---+---+---+---+
		| * | n | h | a |   j   |   u   | d | i | d | h | s | e | v | u |
		+---+---+---+---+-------+-------+---+---+---+---+---+---+---+---+
	*/

	v = (unsigned short) (
		(flipBits[ (v >>  0) & 0x0F ] << 12) |
		(flipBits[ (v >>  4) & 0x0F ] <<  8) |
		(flipBits[ (v >>  8) & 0x0F ] <<  4) |
		(flipBits[ (v >> 12) & 0x0F ] <<  0));

	*(unsigned short*) &attr = v;
#endif
}

void Canonical (HwrDBallEZType& regs)
{
	Canonical (regs.swID);

	Canonical (regs.csAGroupBase);
	Canonical (regs.csBGroupBase);
	Canonical (regs.csCGroupBase);
	Canonical (regs.csDGroupBase);

	Canonical (regs.csASelect);
	Canonical (regs.csBSelect);
	Canonical (regs.csCSelect);
	Canonical (regs.csDSelect);

	Canonical (regs.emuCS);

	Canonical (regs.pllControl);
	Canonical (regs.pllFreqSel);
	Canonical (regs.pllTest);

	Canonical (regs.intControl);
	Canonical (regs.intMaskHi);
	Canonical (regs.intMaskLo);
	Canonical (regs.intStatusHi);
	Canonical (regs.intStatusLo);
	Canonical (regs.intPendingHi);
	Canonical (regs.intPendingLo);

	Canonical (regs.pwmControl);

	Canonical (regs.tmr1Control);
	Canonical (regs.tmr1Prescaler);
	Canonical (regs.tmr1Compare);
	Canonical (regs.tmr1Capture);
	Canonical (regs.tmr1Counter);
	Canonical (regs.tmr1Status);

	Canonical (regs.spiMasterData);
	Canonical (regs.spiMasterControl);

	Canonical (regs.uControl);
	Canonical (regs.uBaud);
	Canonical (regs.uReceive);
	Canonical (regs.uTransmit);
	Canonical (regs.uMisc);
	Canonical (regs.uNonIntPresc);

	Canonical (regs.lcdStartAddr);
	Canonical (regs.lcdScreenWidth);
	Canonical (regs.lcdScreenHeight);
	Canonical (regs.lcdCursorXPos);
	Canonical (regs.lcdCursorYPos);
	Canonical (regs.lcdCursorWidthHeight);
	Canonical (regs.lcdContrastControlPWM);

	Canonical (regs.rtcHourMinSec);
	Canonical (regs.rtcAlarm);
	Canonical (regs.rtcWatchDog);
	Canonical (regs.rtcControl);
	Canonical (regs.rtcIntStatus);
	Canonical (regs.rtcIntEnable);
	Canonical (regs.stopWatch);
	Canonical (regs.rtcDay);
	Canonical (regs.rtcDayAlarm);

	Canonical (regs.dramConfig);
	Canonical (regs.dramControl);

	Canonical (regs.emuAddrCompare);   
	Canonical (regs.emuAddrMask);
	Canonical (regs.emuControlCompare);
	Canonical (regs.emuControlMask);
	Canonical (regs.emuControl);
	Canonical (regs.emuStatus);
}

void Canonical (HwrDBallType& regs)
{
	Canonical (regs.csAGroupBase);
	Canonical (regs.csBGroupBase);
	Canonical (regs.csCGroupBase);
	Canonical (regs.csDGroupBase);

	Canonical (regs.csAGroupMask);
	Canonical (regs.csBGroupMask);
	Canonical (regs.csCGroupMask);
	Canonical (regs.csDGroupMask);

	Canonical (regs.csASelect0);
	Canonical (regs.csASelect1);
	Canonical (regs.csASelect2);
	Canonical (regs.csASelect3);

	Canonical (regs.csBSelect0);
	Canonical (regs.csBSelect1);
	Canonical (regs.csBSelect2);
	Canonical (regs.csBSelect3);

	Canonical (regs.csCSelect0);
	Canonical (regs.csCSelect1);
	Canonical (regs.csCSelect2);
	Canonical (regs.csCSelect3);

	Canonical (regs.csDSelect0);
	Canonical (regs.csDSelect1);
	Canonical (regs.csDSelect2);
	Canonical (regs.csDSelect3);

	Canonical (regs.csDebug);

	Canonical (regs.pllControl);
	Canonical (regs.pllFreqSel);
	Canonical (regs.pllTest);

	Canonical (regs.intControl);
	Canonical (regs.intMaskHi);
	Canonical (regs.intMaskLo);
	Canonical (regs.intWakeupEnHi);
	Canonical (regs.intWakeupEnLo);
	Canonical (regs.intStatusHi);
	Canonical (regs.intStatusLo);
	Canonical (regs.intPendingHi);
	Canonical (regs.intPendingLo);

	Canonical (regs.pwmControl);
	Canonical (regs.pwmPeriod);
	Canonical (regs.pwmWidth);
	Canonical (regs.pwmCounter);

	Canonical (regs.tmr1Control);
	Canonical (regs.tmr1Prescaler);
	Canonical (regs.tmr1Compare);
	Canonical (regs.tmr1Capture);
	Canonical (regs.tmr1Counter);
	Canonical (regs.tmr1Status);
	
	Canonical (regs.tmr2Control);
	Canonical (regs.tmr2Prescaler);
	Canonical (regs.tmr2Compare);
	Canonical (regs.tmr2Capture);
	Canonical (regs.tmr2Counter);
	Canonical (regs.tmr2Status);
	
	Canonical (regs.wdControl);
	Canonical (regs.wdReference);
	Canonical (regs.wdCounter);

	Canonical (regs.spiSlave);

	Canonical (regs.spiMasterData);
	Canonical (regs.spiMasterControl);

	Canonical (regs.uControl);
	Canonical (regs.uBaud);
	Canonical (regs.uReceive);
	Canonical (regs.uTransmit);
	Canonical (regs.uMisc);

	Canonical (regs.lcdStartAddr);
	Canonical (regs.lcdScreenWidth);
	Canonical (regs.lcdScreenHeight);
	Canonical (regs.lcdCursorXPos);
	Canonical (regs.lcdCursorYPos);
	Canonical (regs.lcdCursorWidthHeight);
	Canonical (regs.lcdGrayPalette);

	Canonical (regs.rtcHourMinSec);
	Canonical (regs.rtcAlarm);
	Canonical (regs.rtcReserved);
	Canonical (regs.rtcControl);
	Canonical (regs.rtcIntStatus);
	Canonical (regs.rtcIntEnable);
	Canonical (regs.stopWatch);
}

void Canonical (M68KRegsType& debuggerRegs)
{
	Canonical (debuggerRegs.d[0]);
	Canonical (debuggerRegs.d[1]);
	Canonical (debuggerRegs.d[2]);
	Canonical (debuggerRegs.d[3]);
	Canonical (debuggerRegs.d[4]);
	Canonical (debuggerRegs.d[5]);
	Canonical (debuggerRegs.d[6]);
	Canonical (debuggerRegs.d[7]);

	Canonical (debuggerRegs.a[0]);
	Canonical (debuggerRegs.a[1]);
	Canonical (debuggerRegs.a[2]);
	Canonical (debuggerRegs.a[3]);
	Canonical (debuggerRegs.a[4]);
	Canonical (debuggerRegs.a[5]);
	Canonical (debuggerRegs.a[6]);
//	Canonical (debuggerRegs.a[7]);

	Canonical (debuggerRegs.usp);
	Canonical (debuggerRegs.ssp);
	Canonical (debuggerRegs.sr);
	Canonical (debuggerRegs.pc);
}


void Canonical (Mem1ChunkHeaderType& p)
{
	Canonical (p.size);
	Canonical (p.lockOwner);
	Canonical (p.flags);
	Canonical (p.hOffset);
}


void Canonical (Mem1HeapHeaderType& p)
{
	Canonical (p.flags);
	Canonical (p.size);
	Canonical (p.mstrPtrTbl);
}


void Canonical (Mem1MstrPtrTableType& p)
{
	Canonical (p.numEntries);
	Canonical (p.nextTblOffset);
//	Canonical (p.mstrP);	// !!! Swap the table?
}


void Canonical (Mem2HeapHeaderType& p)
{
	Canonical (p.flags);
	Canonical (p.size);
	Canonical (p.mstrPtrTbl);
}


void Canonical (MemChunkHeaderType& p)
{
	DWord	size = p.size << 8;
	Canonical (size);
	p.size = size;

	DWord	hOffset = p.hOffset << 8;
	Canonical (hOffset);	// offset/2 to handle for movables
	p.hOffset = hOffset;
}


void Canonical (MemMstrPtrTableType& p)
{
	Canonical (p.numEntries);
	Canonical (p.nextTblOffset);
//	Canonical (p.mstrP);	// !!! Swap the table?
}


void Canonical (PenBtnInfoType& p)
{
	Canonical (p.boundsR);
	Canonical (p.asciiCode);
	Canonical (p.keyCode);
	Canonical (p.modifiers);
}


void Canonical (Point& p)
{
	Canonical (p.h);
	Canonical (p.v);
}


void Canonical (PointType& p)
{
	Canonical (p.x);
	Canonical (p.y);
}


void Canonical (RecordListType& recordList)
{
	Canonical (recordList.nextRecordListID);
	Canonical (recordList.numRecords);
	Canonical (recordList.firstEntry);
}


void Canonical (RecordEntryType& entry)
{
	Canonical (entry.localChunkID);
	Canonical (entry.attributes);
//	Canonical (entry.uniqueID);	// array of 3 bytes
}


void Canonical (RectangleType& r)
{
	Canonical (r.topLeft);
	Canonical (r.extent);
}


void Canonical (RsrcEntryType& entry)
{
	Canonical (entry.type);
	Canonical (entry.id);
	Canonical (entry.localChunkID);
}


void Canonical (SlkPktHeaderType& hdr)
{
	Canonical (hdr.signature1);
	Canonical (hdr.signature2);
	Canonical (hdr.dest);
	Canonical (hdr.src);
	Canonical (hdr.type);
	Canonical (hdr.bodySize);
	Canonical (hdr.transId);
	Canonical (hdr.checksum);
}


void Canonical (SlkPktFooterType& ftr)
{
	Canonical (ftr.crc16);
}


void Canonical (SysAppInfoType& info)
{
	Canonical (info.cmd);
	Canonical (info.cmdPBP);
	Canonical (info.launchFlags);
	Canonical (info.taskID);
	Canonical (info.codeH);
	Canonical (info.dbP);
	Canonical (info.stackP);
	Canonical (info.globalsChunkP);
	Canonical (info.memOwnerID);
	Canonical (info.dmAccessP);
	Canonical (info.dmLastErr);
	Canonical (info.errExceptionP);
	Canonical (info.a5Ptr);
	Canonical (info.stackEndP);
	Canonical (info.globalEndP);
	Canonical (info.rootP);
	Canonical (info.extraP);
}


void Canonical (SysKernelInfoType& info)
{
	// Compile time assertion
	{
		char	_dummy[sizeof (info.selector) == 1 ? 1 : 0];
		(void)_dummy;
	}

	Canonical (info.selector);
	Canonical (info.id);

	switch (info.selector)
	{
		case sysKernelInfoSelCurTaskInfo:
		case sysKernelInfoSelTaskInfo:
			Canonical (info.param.task.id);
			Canonical (info.param.task.nextID);
			Canonical (info.param.task.tag);
			Canonical (info.param.task.status);
			Canonical (info.param.task.timer);
			Canonical (info.param.task.timeSlice);
			Canonical (info.param.task.priority);
			Canonical (info.param.task.attributes);
			Canonical (info.param.task.pendingCalls);
			Canonical (info.param.task.senderTaskID);
			Canonical (info.param.task.msgExchangeID);
			Canonical (info.param.task.tcbP);
			Canonical (info.param.task.stackP);
			Canonical (info.param.task.stackStart);
			Canonical (info.param.task.stackSize);
			break;

		case sysKernelInfoSelSemaphoreInfo:
			Canonical (info.param.semaphore.id);
			Canonical (info.param.semaphore.nextID);
			Canonical (info.param.semaphore.tag);
			Canonical (info.param.semaphore.initValue);
			Canonical (info.param.semaphore.curValue);
			Canonical (info.param.semaphore.nestLevel);
			Canonical (info.param.semaphore.ownerID);
			break;

		case sysKernelInfoSelTimerInfo:
			Canonical (info.param.timer.id);
			Canonical (info.param.timer.nextID);
			Canonical (info.param.timer.tag);
			Canonical (info.param.timer.ticksLeft);
			Canonical (info.param.timer.period);
			Canonical (info.param.timer.proc);
			break;
	}
}


void Canonical (SysNVParamsType& params)
{
	Canonical (params.rtcHours);
	Canonical (params.rtcHourMinSecCopy);
}


void Canonical (SysPktBodyType& packet)
{
	// Compile time assertion
	{
		char	_dummy[sizeof (packet.command) == 1 ? 1 : 0];
		(void)_dummy;
	}

	Canonical (packet.command);

	switch (packet.command)
	{
		case sysPktStateCmd:
			Canonical ((SysPktStateCmdType&) packet);
			break;

		case sysPktStateRsp:
			Canonical ((SysPktStateRspType&) packet);
			break;

		case sysPktReadMemCmd:
			Canonical ((SysPktReadMemCmdType&) packet);
			break;

		case sysPktReadMemRsp:
			Canonical ((SysPktReadMemRspType&) packet);
			break;

		case sysPktWriteMemCmd:
			Canonical ((SysPktWriteMemCmdType&) packet);
			break;

		case sysPktWriteMemRsp:
			Canonical ((SysPktWriteMemRspType&) packet);
			break;

		case sysPktSingleStepCmd:
			// I don't think there's anything to do here.  I think that
			// single-stepping is performed by the debugger setting the
			// trace bit in the SR and sending a sysPktContinueCmd.
			break;

		case sysPktGetRtnNameCmd:
			Canonical ((SysPktRtnNameCmdType&) packet);
			break;

		case sysPktGetRtnNameRsp:
			Canonical ((SysPktRtnNameRspType&) packet);
			break;

		case sysPktReadRegsCmd:
			Canonical ((SysPktReadRegsCmdType&) packet);
			break;

		case sysPktReadRegsRsp:
			Canonical ((SysPktReadRegsRspType&) packet);
			break;

		case sysPktWriteRegsCmd:
			Canonical ((SysPktWriteRegsCmdType&) packet);
			break;

		case sysPktWriteRegsRsp:
			Canonical ((SysPktWriteRegsRspType&) packet);
			break;

		case sysPktContinueCmd:
			Canonical ((SysPktContinueCmdType&) packet);
			break;

		case sysPktRPCCmd:
		case sysPktRPCRsp:
			Canonical ((SysPktRPCType&) packet);
			break;

		case sysPktGetBreakpointsCmd:
			Canonical ((SysPktGetBreakpointsCmdType&) packet);
			break;

		case sysPktGetBreakpointsRsp:
			Canonical ((SysPktGetBreakpointsRspType&) packet);
			break;

		case sysPktSetBreakpointsCmd:
			Canonical ((SysPktSetBreakpointsCmdType&) packet);
			break;

		case sysPktSetBreakpointsRsp:
			Canonical ((SysPktSetBreakpointsRspType&) packet);
			break;

//		case sysPktRemoteUIUpdCmd:
//			// Sent TO debugger; never received FROM debugger.
//			break;

//		case sysPktRemoteEvtCmd:
//			// Sent TO debugger; never received FROM debugger.
//			break;

		case sysPktDbgBreakToggleCmd:
			Canonical ((SysPktDbgBreakToggleCmdType&) packet);
			break;

		case sysPktDbgBreakToggleRsp:
			Canonical ((SysPktDbgBreakToggleRspType&) packet);
			break;

		case sysPktFlashCmd:
		case sysPktFlashRsp:
			Canonical ((SysPktFlashWriteType&) packet);
			break;

		case sysPktCommCmd:
			Canonical ((SysPktCommCmdType&) packet);
			break;

		case sysPktCommRsp:
			Canonical ((SysPktCommRspType&) packet);
			break;

		case sysPktGetTrapBreaksCmd:
			Canonical ((SysPktGetTrapBreaksCmdType&) packet);
			break;

		case sysPktGetTrapBreaksRsp:
			Canonical ((SysPktGetTrapBreaksRspType&) packet);
			break;

		case sysPktSetTrapBreaksCmd:
			Canonical ((SysPktSetTrapBreaksCmdType&) packet);
			break;

		case sysPktSetTrapBreaksRsp:
			Canonical ((SysPktSetTrapBreaksRspType&) packet);
			break;

		case sysPktGremlinsCmd:
			Canonical ((SysPktGremlinsCmdType&) packet);
			break;

		case sysPktFindCmd:
			Canonical ((SysPktFindCmdType&) packet);
			break;

		case sysPktFindRsp:
			Canonical ((SysPktFindRspType&) packet);
			break;

		case sysPktGetTrapConditionsCmd:
			Canonical ((SysPktGetTrapConditionsCmdType&) packet);
			break;

		case sysPktGetTrapConditionsRsp:
			Canonical ((SysPktGetTrapConditionsRspType&) packet);
			break;

		case sysPktSetTrapConditionsCmd:
			Canonical ((SysPktSetTrapConditionsCmdType&) packet);
			break;

		case sysPktSetTrapConditionsRsp:
			Canonical ((SysPktSetTrapConditionsRspType&) packet);
			break;

		case sysPktChecksumCmd:
		case sysPktChecksumRsp:
			Canonical ((SysPktChecksumType&) packet);
			break;

		case sysPktExecFlashCmd:
		case sysPktExecFlashRsp:
			Canonical ((sysPktExecFlashType&) packet);
			break;

		case sysPktRemoteMsgCmd:
			Canonical ((SysPktRemoteMsgCmdType&) packet);
			break;

		default:
			break;
	}
}


void Canonical (SysPktChecksumType& packet)
{
	Canonical (packet.command);

	Canonical (packet.firstAddr);
	Canonical (packet.numBytes);
	Canonical (packet.seed);
	Canonical (packet.checksum);
}


void Canonical (SysPktCommCmdType& packet)
{
	Canonical (packet.command);

	Canonical (packet.set);
	Canonical (packet.baudRate);
	Canonical (packet.flags);
}


void Canonical (SysPktCommRspType& packet)
{
	Canonical (packet.command);

	Canonical (packet.baudRate);
	Canonical (packet.flags);
}


void Canonical (SysPktContinueCmdType& packet)
{
	Canonical (packet.command);

	Canonical (packet.regs);
	Canonical (packet.stepSpy);
	Canonical (packet.ssAddr);
	Canonical (packet.ssCount);
	Canonical (packet.ssCheckSum);
}


void Canonical (SysPktDbgBreakToggleCmdType& packet)
{
	Canonical (packet.command);
}


void Canonical (SysPktDbgBreakToggleRspType& packet)
{
	Canonical (packet.command);

	Canonical (packet.newState);
}


void Canonical (sysPktExecFlashType& packet)
{
	Canonical (packet.command);

	Canonical (packet.numBytes);
	Canonical (packet.flashBase);
	Canonical (packet.srcAddr);
	Canonical (packet.dstAddr);
	Canonical (packet.codeAddr);
	Canonical (packet.flashResID);
	Canonical (packet.manuf);
	Canonical (packet.device);
	Canonical (packet.devInfo);
	Canonical (packet.remoteCmd);
	Canonical (packet.response);
	Canonical (packet.spareByte);
	Canonical (packet.spareDWord);
}


void Canonical (SysPktFindCmdType& packet)
{
	Canonical (packet.command);

	Canonical (packet.firstAddr);
	Canonical (packet.lastAddr);
	Canonical (packet.numBytes);
	Canonical (packet.caseInsensitive);
}


void Canonical (SysPktFindRspType& packet)
{
	Canonical (packet.command);

	Canonical (packet.addr);
	Canonical (packet.found);
}


void Canonical (SysPktFlashWriteType& packet)
{
	Canonical (packet.command);

	Canonical (packet.numBytes);
	Canonical (packet.flashBase);
	Canonical (packet.srcAddr);
	Canonical (packet.dstAddr);
	Canonical (packet.manuf);
	Canonical (packet.device);
	Canonical (packet.width);
	Canonical (packet.filler);
	Canonical (packet.ramSpaceP);

}


void Canonical (SysPktGetBreakpointsCmdType& packet)
{
	Canonical (packet.command);
}


void Canonical (SysPktGetBreakpointsRspType& packet)
{
	SysPktGetBreakpointsRspType2&	packet2 = (SysPktGetBreakpointsRspType2&) packet;

	Canonical (packet2.command);

	for (int ii = 0; ii < dbgTotalBreakpoints; ++ii)
		Canonical (packet2.bp[ii]);
}


void Canonical (SysPktGetTrapBreaksCmdType& packet)
{
	Canonical (packet.command);
}


void Canonical (SysPktGetTrapBreaksRspType& packet)
{
	SysPktGetTrapBreaksRspType2&	packet2 = (SysPktGetTrapBreaksRspType2&) packet;

	Canonical (packet2.command);

	for (int ii = 0; ii < dbgTotalTrapBreaks; ++ii)
		Canonical (packet2.trapBP[ii]);
}


void Canonical (SysPktGetTrapConditionsCmdType& packet)
{
	Canonical (packet.command);
}


void Canonical (SysPktGetTrapConditionsRspType& packet)
{
	SysPktGetTrapConditionsRspType2&	packet2 = (SysPktGetTrapConditionsRspType2&) packet;

	Canonical (packet2.command);

	for (int ii = 0; ii < dbgTotalTrapBreaks; ++ii)
		Canonical (packet2.trapParam[ii]);
}


void Canonical (SysPktGremlinsCmdType& packet)
{
	Canonical (packet.command);
	Canonical (packet.action);

	// We can't really decode the data unless we know the
	// action number.  And we don't know if we know the
	// action number because we don't know if it is in
	// host or canonical format.
}


void Canonical (SysPktReadMemCmdType& packet)
{
	Canonical (packet.command);

	Canonical (packet.address);
	Canonical (packet.numBytes);
}


void Canonical (SysPktReadMemRspType& packet)
{
	Canonical (packet.command);

	// Variable length data follows.  This is all just a
	// stream of bytes; it can't be byteswapped without
	// knowing the format that the requester will apply
	// to those bytes.
}


void Canonical (SysPktReadRegsCmdType& packet)
{
	Canonical (packet.command);
}


void Canonical (SysPktReadRegsRspType& packet)
{
	SysPktReadRegsRspType2&	packet2 = (SysPktReadRegsRspType2&) packet;

	Canonical (packet2.command);

	Canonical (packet2.reg);
}


void Canonical (SysPktRemoteMsgCmdType& packet)
{
	Canonical (packet.command);

	// The rest of this guy is just straight text.
}


void Canonical (SysPktRPCType& packet)
{
	Canonical (packet.command);

	Canonical (packet.trapWord);
	Canonical (packet.resultD0);
	Canonical (packet.resultA0);
	Canonical (packet.numParams);

	// Nothing really to do with the "param" field.  "byRef" and "size"
	// are bytes, so nothing to swap there, and the "data" field is
	// of indeterminate size, so we can't bytewap it either.  That means
	// that we must take care to always treat it as a big-endian value.	
}


void Canonical (SysPktRtnNameCmdType& packet)
{
	Canonical (packet.command);

	Canonical (packet.address);
}


void Canonical (SysPktRtnNameRspType& packet)
{
	Canonical (packet.command);

	Canonical (packet.address);
	Canonical (packet.startAddr);
	Canonical (packet.endAddr);
}


void Canonical (SysPktSetBreakpointsCmdType& packet)
{
	SysPktSetBreakpointsCmdType2&	packet2 = (SysPktSetBreakpointsCmdType2&) packet;

	Canonical (packet2.command);

	for (int ii = 0; ii < dbgTotalBreakpoints; ++ii)
		Canonical (packet2.bp[ii]);
}


void Canonical (SysPktSetBreakpointsRspType& packet)
{
	Canonical (packet.command);
}


void Canonical (SysPktSetTrapBreaksCmdType& packet)
{
	SysPktSetTrapBreaksCmdType2&	packet2 = (SysPktSetTrapBreaksCmdType2&) packet;

	Canonical (packet2.command);

	for (int ii = 0; ii < dbgTotalTrapBreaks; ++ii)
		Canonical (packet2.trapBP[ii]);
}


void Canonical (SysPktSetTrapBreaksRspType& packet)
{
	Canonical (packet.command);
}


void Canonical (SysPktSetTrapConditionsCmdType& packet)
{
	SysPktSetTrapConditionsCmdType2&	packet2 = (SysPktSetTrapConditionsCmdType2&) packet;

	Canonical (packet2.command);

	for (int ii = 0; ii < dbgTotalTrapBreaks; ++ii)
		Canonical (packet2.trapParam[ii]);
}


void Canonical (SysPktSetTrapConditionsRspType& packet)
{
	Canonical (packet.command);
}


void Canonical (SysPktStateCmdType& packet)
{
	Canonical (packet.command);
}


void Canonical (SysPktStateRspType& packet)
{
	Canonical (packet.command);

	Canonical (packet.resetted);
	Canonical (packet.exceptionId);
	Canonical (packet.reg);

	int ii;
	for (ii = 0; ii < dbgTotalBreakpoints; ++ii)
		Canonical (packet.bp[ii]);

	for (ii = 0; ii < sysPktStateRspInstWords; ++ii)
		Canonical (packet.inst[ii]);

	Canonical (packet.startAddr);
	Canonical (packet.endAddr);
	Canonical (packet.trapTableRev);	
}


void Canonical (SysPktWriteRegsCmdType& packet)
{
	Canonical (packet.command);

	Canonical (packet.reg);
}


void Canonical (SysPktWriteRegsRspType& packet)
{
	Canonical (packet.command);
}


void Canonical (SysPktWriteMemCmdType& packet)
{
	Canonical (packet.command);

	Canonical (packet.address);
	Canonical (packet.numBytes);
}


void Canonical (SysPktWriteMemRspType& packet)
{
	Canonical (packet.command);
}


void Canonical (regstruct& p)
{
	Canonical (p.regs[0]);
	Canonical (p.regs[1]);
	Canonical (p.regs[2]);
	Canonical (p.regs[3]);
	Canonical (p.regs[4]);
	Canonical (p.regs[5]);
	Canonical (p.regs[6]);
	Canonical (p.regs[7]);

	Canonical (p.regs[8]);
	Canonical (p.regs[9]);
	Canonical (p.regs[10]);
	Canonical (p.regs[11]);
	Canonical (p.regs[12]);
	Canonical (p.regs[13]);
	Canonical (p.regs[14]);
	Canonical (p.regs[15]);

	Canonical (p.usp);
	Canonical (p.isp);
	Canonical (p.msp);
	Canonical (p.sr);
	Canonical (p.t1);
	Canonical (p.t0);
	Canonical (p.s);
	Canonical (p.m);
	Canonical (p.stopped);
	Canonical (p.intmask);
	Canonical (p.pc);
	Canonical (p.pc_p);
	Canonical (p.pc_oldp);
	Canonical (p.pc_meta_oldp);
	Canonical (p.vbr);
	Canonical (p.sfc);
	Canonical (p.dfc);

#if 0	// we don't support FP
	Canonical (fp.regs[0]);
	Canonical (fp.regs[1]);
	Canonical (fp.regs[2]);
	Canonical (fp.regs[3]);
	Canonical (fp.regs[4]);
	Canonical (fp.regs[5]);
	Canonical (fp.regs[6]);
	Canonical (fp.regs[7]);

	Canonical (p.fpcr);
	Canonical (p.fpsr);
	Canonical (p.fpiar);
#endif

	Canonical (p.spcflags);
	Canonical (p.kick_mask);
	Canonical (p.prefetch);
}



#endif	// BYTESWAP
