/*******************************************************************
 * 							 Pilot Software
 *
 *		 Copyright(c) 1998, Palm Computing Inc., All Rights Reserved
 *
 *-------------------------------------------------------------------
 * FileName:
 *		PPCMgr.h
 *
 * Description:
 *		SerialMgr -> PPCToolbox mapper Routines
 *
 *		These functions are called from Transport.c if we're
 *		communicating via the PPCToolbox instead of an actual
 *		serial connection.
 *
 * History:
 *		6/30/95  kaar - Created
 *
 *******************************************************************/

#ifndef __PPCMGR_H
#define __PPCMGR_H


#ifdef __cplusplus
extern "C" {
#endif

// Acquires and opens a serial port with given baud and default settings.
Err PPPCOpen(void);

// Closes the serial connection previously opened with PPPCOpen.
Err PPPCClose(void);

// Reset error condition of serial connection
Err PPCClearErr(void);

// Waits until the serial transmit buffer empties.
// The timeout arg is ignored; CTS timeout is used
Err PPCSendWait(Long timeout);

// Waits for at least 'bytes' bytes of data to arrive at the serial input.
//  but does not read them in
Err PPCReceiveWait(ULong bytes, Long timeout);

// Flushes any data coming into the serial port, discarding the data.
void PPCReceiveFlush(Long timeout);

// Specify a new input buffer.  To restore the original buffer, pass
// bufSize = 0.
Err PPCSetReceiveBuffer(VoidPtr bufP, UInt bufSize);

// Sends a buffer of data (may queue it up and return).
ULong PPCSend(VoidPtr bufP, ULong count, Err* errP);

// Receives a buffer of data of the given size.
ULong PPCReceive(VoidPtr bufP, ULong count, Long timeout, Err* errP);


#ifdef __cplusplus
}
#endif

#endif	// __PPCMGR_H